# Android makefile for ffmpeg libavcodec

LOCAL_PATH := $(call my-dir)

include $(CLEAR_VARS)

include $(LOCAL_PATH)/../ffmpeg.mk

LOCAL_MODULE    := libavcodec

LOCAL_C_INCLUDES :=		\
	$(LOCAL_PATH)		\
	$(LOCAL_PATH)/..

LOCAL_PATH := $(call my-dir)/Sources/libavcodec

#### COMMONS #####

# ffmpeg libavcodec common
LOCAL_SRC_FILES := allcodecs.c audioconvert.c avpacket.c bitstream.c bitstream_filter.c dsputil.c faanidct.c imgconvert.c jrevdct.c opt.c options.c parser.c raw.c resample.c resample2.c simple_idct.c utils.c

# ffmpeg libavcodec codecs parts
LOCAL_SRC_FILES += dct.c avfft.c fft.c mdct.c rdft.c h264dsp.c h264idct.c h264pred.c lsp.c golomb.c

#### DSP/ARM SUPPORT #####

# Basic ARM support
LOCAL_SRC_FILES += arm/dsputil_init_arm.c arm/dsputil_arm.S arm/fft_init_arm.c arm/jrevdct_arm.S arm/mpegvideo_arm.c arm/simple_idct_arm.S arm/h264dsp_init_arm.c arm/h264pred_init_arm.c

# ARMV5TE support
LOCAL_SRC_FILES += arm/dsputil_init_armv5te.c arm/mpegvideo_armv5te.c arm/mpegvideo_armv5te_s.S arm/simple_idct_armv5te.S

#### CODECS #####

# AAC
LOCAL_SRC_FILES += aacdec.c aactab.c aacsbr.c aacps.c aacadtsdec.c mpeg4audio.c

# COOK
LOCAL_SRC_FILES += cook.c

# H263
LOCAL_SRC_FILES += h263dec.c h263.c ituh263dec.c mpeg4video.c mpeg4videodec.c flvdec.c intelh263dec.c mpegvideo.c error_resilience.c

# MP3 (+ADU, +ON-MP4)
LOCAL_SRC_FILES += mpegaudiodec.c mpegaudiodecheader.c mpegaudio.c mpegaudiodata.c mpeg4audio.c

# RA (144 & 288)
LOCAL_SRC_FILES += ra144.c ra144dec.c celp_filters.c ra288.c celp_math.c

# SQV3
LOCAL_SRC_FILES += h264.c svq3.c h264_loopfilter.c h264_direct.c h264_sei.c h264_ps.c h264_refs.c h264_cavlc.c h264_cabac.c cabac.c mpegvideo.c error_resilience.c svq1dec.c svq1.c h263.c

# WMA V1, V2, Voice, Pro
LOCAL_SRC_FILES += wmadec.c wmaprodec.c wmavoice.c wma.c aactab.c celp_math.c celp_filters.c acelp_vectors.c acelp_filters.c

# VORBIS
LOCAL_SRC_FILES += vorbis_dec.c vorbis.c vorbis_data.c xiph.c

#### DEMUXERS DEPENDENCIES #####

# MOV
LOCAL_SRC_FILES += mpeg4audio.c mpegaudiodata.c

# OGG
LOCAL_SRC_FILES += flacdec.c flacdata.c flac.c dirac.c mpeg12data.c

#### PARSERS #####

# AAC (+LATM)
LOCAL_SRC_FILES += aac_parser.c aac_ac3_parser.c aacadtsdec.c mpeg4audio.c latm_parser.c

# H263
LOCAL_SRC_FILES += h263_parser.c

# H264
LOCAL_SRC_FILES += h264_parser.c h264.c cabac.c h264_refs.c h264_sei.c h264_direct.c h264_loopfilter.c h264_cabac.c h264_cavlc.c h264_ps.c mpegvideo.c error_resilience.c

# MPEG4 Video
LOCAL_SRC_FILES += mpeg4video_parser.c h263.c mpegvideo.c error_resilience.c mpeg4videodec.c mpeg4video.c ituh263dec.c h263dec.c

# MPEG Audio
LOCAL_SRC_FILES += mpegaudio_parser.c mpegaudiodecheader.c mpegaudiodata.c

#### BITSTREAM FILTERS #####

# AAC ADTS to ASC
LOCAL_SRC_FILES += aac_adtstoasc_bsf.c aacadtsdec.c mpeg4audio.c

LOCAL_CFLAGS += $(FFCFLAGS) -I$(LOCAL_PATH) -I$(LOCAL_PATH)/..

LOCAL_STATIC_LIBRARIES := $(FFMPEG_LIBS)

include $(BUILD_STATIC_LIBRARY)

