/*=============================================================================
* Copyright (c) 2008-2009, BARACODA MEDIA
* All Rights Reserved
*
*
* The copyright notice above does not evidence any
* actual or intended publication of such source code.
===============================================================================
* Description:
*     PhoenixAudioPlayer Java API
*
===============================================================================
*
*     Created By    : Tarik Litim
*     Creation Date : 26/05/2010
*     Version       : 1.0
*
*	  $LastChangedDate$
*     $LastChangedBy$
*     $LastChangedRevision$
*
*     $Id$
*
===============================================================================
* History
*     1.0    26/05/2010   Component creation
*     1.1    15/03/2011   API Update
==============================================================================*/

package com.baracodamedia.phaudioplayer.api;

import android.media.*;
import android.util.Log;

public class phaudioplayer extends Thread
{
	static {
		System.loadLibrary("phaplayer-jni");
	}
	
	private AudioTrack AudioDSP;
	private byte[] AudioBuffer;
	private int AudioBufferSize = 32768;
	private boolean IsPaused=false;
	
	/* Native network connection states */
	private final byte Native_phapnsConnecting=0; /**< Connecting. */
	private final byte Native_phapnsFailed=1; /**< Failed. */
	private final byte Native_phapnsConnected=2; /**< Connected. */
	private final byte Native_phapnsTimeout=3; /**< Timeout. */
	private final byte Native_phapnsDisconnected=4; /**< Disconnected. */
	private final byte Native_phapnsUnsupportedProtocol=5; /**< Unsupported transport protocol. */
	private final byte Native_phapnsDownloading=6; /**< Downloading. */
	private final byte Native_phapnsFailedParsingUrl=7; /**< Error: parsing the URI. */
	private final byte Native_phapnsHostNotFound=8; /** Error: Host not found. */
	private final byte Native_phapnsUnsupportedPlaylistUrl=9; /** Error: Unsupported playlist parsing. */
	private final byte Native_phapnsHostDenied=10; /** Error: Host denied connection. */
	private final byte Native_phapnsUnsupportedContent=11; /** Error: Unsupported server content. */
	
	/* Native audio playing states */
	private final byte Native_phapsBuffering=0; /**< Buffering. */
	private final byte Native_phapsPlay=1; /**< Playing. */
	private final byte Native_phapsStop=2; /**< Stopped. */
	private final byte Native_phapsSeek=3; /**< Seeking. */
	private final byte Native_phapsPause=4; /**< Pausing. */
	private final byte Native_phapsError=5; /**< Error. */
	private final byte Native_phapsUnsupportedFormat=6; /**< Unsupported format.  */
	private final byte Native_phapsSync=7; /**< Synching.  */
	
	/* Native audio debug layers */	
	public static int PHOENIX_DEBUG_LAYER_NONE=0x00;
	public static int PHOENIX_DEBUG_LAYER_NETWORK=0x01;
	public static int PHOENIX_DEBUG_LAYER_AUDIOPLAYER=0x06;
	public static int PHOENIX_DEBUG_LAYER_AUDIODECODER=0x07;
	public static int PHOENIX_DEBUG_LAYER_AUDIOBYTESTREAMIO=0x08;
	public static int PHOENIX_DEBUG_LAYER_AUDIOMANAGER=0x0d;
	public static int PHOENIX_DEBUG_LAYER_PLATFORM=0x0f;
	public static int PHOENIX_DEBUG_LAYER_JNIWRAPPER=0x10;
	public static int PHOENIX_DEBUG_LAYER_IPC=0x11;
	public static int PHOENIX_DEBUG_LAYER_ALL=0xff;
	
	/* Native audio debug verbosity level */
	public static int PHOENIX_DEBUG_LEVEL_ERROR=0x01; /**< Errors only. */
	public static int PHOENIX_DEBUG_LEVEL_INFO=0x02; /**< Info & Errors. */
	public static int PHOENIX_DEBUG_LEVEL_VERBOSE=0x03; /**< Verbose, info & errors. */
	
	/* Audio Player Listener */
	public phaudioplayerlistener PhAListener;
    
    /* Native Audio Player API */
    public native boolean Play(String AURL);
    public native boolean Stop();
    public native boolean Pause();
    public native boolean Sync();
    public native boolean Seek(int ASecondsToStart);
    
    public native void SetDebugLayer(int ALayer, int ALevel, int AEnable);
    
    /* Private Audio HAL */
    public void AudioHwInit(){
    	Log.v("[phaudioplayer]", "AudioHwInit");
    	
    	/* Nothing to do...*/
    }
    public byte[] AudioHwOpen(int AChannels, int ASampleRate){

    	Log.v("[phaudioplayer]", "AudioHwOpen(" + AChannels + "," + ASampleRate + ")");
    	
    	//int LBufSize = AudioTrack.getMinBufferSize(ASampleRate, AudioFormat.CHANNEL_CONFIGURATION_STEREO,AudioFormat.ENCODING_PCM_16BIT);
    	
    	// To reduce JNI overheard calls!
    	int LBufSize = AudioBufferSize;
    	IsPaused = false;
    	
    	Log.v("[phaudioplayer]", "Minimal Block Size=" + LBufSize);
    	
    	AudioBuffer = new byte[LBufSize];
    	
    	AudioDSP = new AudioTrack(AudioManager.STREAM_MUSIC, ASampleRate, AudioFormat.CHANNEL_CONFIGURATION_STEREO,
    			AudioFormat.ENCODING_PCM_16BIT , LBufSize, AudioTrack.MODE_STREAM);
    	
    	AudioDSP.play();
    	
    	return AudioBuffer;
    }
    
    public void AudioHwClose(){
    	Log.v("[phaudioplayer]", "AudioHwClose()");
    	
    	if(AudioDSP != null)
    	AudioDSP.stop();
    	
    	AudioDSP = null;
    	AudioBuffer = null;
    }
    
    public void AudioHwReset(){
    	Log.v("[phaudioplayer]", "AudioHwReset()");
    	
    	if(AudioDSP != null)
    	AudioDSP.flush();
    }
    
   public int AudioHwWrite(){
    	int LWritten=0;
    	
    	//Log.v("[phaudioplayer]", "==> AudioHwWrite(" + AudioBuffer.length + ")");
    	
    	if(AudioDSP != null)
    	LWritten = AudioDSP.write(AudioBuffer, 0, AudioBuffer.length);
    	
    	//Log.v("[phaudioplayer]", "<== AudioHwWrite() Written=" +LWritten);
    	
    	return LWritten;
    }
   
   public void AudioHwPause(){
   	Log.v("[phaudioplayer]", "AudioHwPause()");
   	
   	if(AudioDSP != null)
   	{
   		// Stopped HERE!! Need Pause()
   		if(!IsPaused)
   		{
   			IsPaused = true;
   			AudioDSP.pause();
   		}
   		else
   		{
   			
   			AudioDSP.play();
   		}
   	}
   	
   }
    
    
    /*
     * Callbacks
     */
    public void OnNetworkStatusEvent(int ANetworkStatus){
    	
    	PhAPlayNetworkEvtType LNetworkStatus;
    	
		Log.v("[phaudioplayer]", "OnNetworkStatusEvent()=" + ANetworkStatus);
		
		switch(ANetworkStatus)
		{
			case Native_phapnsConnecting:
				LNetworkStatus = PhAPlayNetworkEvtType.phapnsConnecting;
				break;
			case Native_phapnsFailed:
				LNetworkStatus = PhAPlayNetworkEvtType.phapnsFailed;
				break;
			case Native_phapnsConnected:
				LNetworkStatus = PhAPlayNetworkEvtType.phapnsConnected;
				break;
			case Native_phapnsTimeout:
				LNetworkStatus = PhAPlayNetworkEvtType.phapnsTimeout;
				break;
			case Native_phapnsDisconnected:
				LNetworkStatus = PhAPlayNetworkEvtType.phapnsDisconnected;
				break;
			case Native_phapnsUnsupportedProtocol:
				LNetworkStatus = PhAPlayNetworkEvtType.phapnsUnsupportedProtocol;
				break;
			case Native_phapnsDownloading:
				LNetworkStatus = PhAPlayNetworkEvtType.phapnsDownloading;
				break;
			case Native_phapnsFailedParsingUrl:
				LNetworkStatus = PhAPlayNetworkEvtType.phapnsFailedParsingUrl;
				break;
			case Native_phapnsHostNotFound:
				LNetworkStatus = PhAPlayNetworkEvtType.phapnsHostNotFound;
				break;
			case Native_phapnsUnsupportedPlaylistUrl:
				LNetworkStatus = PhAPlayNetworkEvtType.phapnsUnsupportedPlaylistUrl;
				break;
			case Native_phapnsHostDenied:
				LNetworkStatus = PhAPlayNetworkEvtType.phapnsHostDenied;
				break;

			case Native_phapnsUnsupportedContent:
				LNetworkStatus = PhAPlayNetworkEvtType.phapnsUnsupportedContent;
				break;

			default:
					Log.v("[phaudioplayer]", "Unknown status=" + ANetworkStatus);
					return;
		}
				
		if(PhAListener != null)
		PhAListener.OnNetworkEvent(LNetworkStatus);		
    	
    }
    
    public void OnPlayStatusEvent(int APlayStatus){
    	Log.v("[phaudioplayer]", "OnPlayStatusEvent()=" + APlayStatus);
    	
    	PhAPlayEvtType LPlayStatus;
		
		switch(APlayStatus)
		{
		case Native_phapsBuffering: 
			LPlayStatus = PhAPlayEvtType.phapsBuffering;
			break;
		case Native_phapsPlay:
			LPlayStatus = PhAPlayEvtType.phapsPlay;
			break;
		case Native_phapsStop:
			LPlayStatus = PhAPlayEvtType.phapsStop;
			break;
		case Native_phapsSeek:
			LPlayStatus = PhAPlayEvtType.phapsSeek;
			break;
		case Native_phapsPause:
			LPlayStatus = PhAPlayEvtType.phapsPause;
			break;
		case Native_phapsError:
			LPlayStatus = PhAPlayEvtType.phapsError;
			break;
		case Native_phapsSync:
			LPlayStatus = PhAPlayEvtType.phapsSync;
			break;
		case Native_phapsUnsupportedFormat:
			LPlayStatus = PhAPlayEvtType.phapsUnsupportedFormat;
			break;
			
		default:
			Log.v("[phaudioplayer]", "Unknown status=" + APlayStatus);
			return;
		}
		
		if(PhAListener != null)
		PhAListener.OnPlayEvent(LPlayStatus);
    	
    }

    public void OnPlayMediaInfoEvent(String Codec, String Title, String Access, int Channels, int Bitrate, 
    		int SampleRate, int StartPosition, int Duration, int FileSize){
    	Log.v("[phaudioplayer]", "OnPlayMediaInfoEvent()");
    	Log.v("[phaudioplayer]", "Codec=" + Codec);
    	Log.v("[phaudioplayer]", "Title=" + Title);
    	Log.v("[phaudioplayer]", "Access=" + Access);
    	Log.v("[phaudioplayer]", "Channels=" + Channels);
    	Log.v("[phaudioplayer]", "Bitrate=" + Bitrate);
    	Log.v("[phaudioplayer]", "SampleRate=" + SampleRate);
    	Log.v("[phaudioplayer]", "StartPosition=" + StartPosition);
    	Log.v("[phaudioplayer]", "Duration=" + Duration);
    	Log.v("[phaudioplayer]", "FileSize=" + FileSize);
    	
    	if(PhAListener != null)
    	PhAListener.OnPlayMediaInfoEvent(Codec, Title, Access, Channels, Bitrate, SampleRate, StartPosition, Duration, FileSize);
    	
    }
    
    public void OnBufferStatusEvent(int ABufferStatus){
    	Log.v("[phaudioplayer]", "OnBufferStatusEvent=" + ABufferStatus);
    	
    	/* Disabled on native side, produces too much overhead and btw useless... */
    	
    	if(PhAListener != null)
    	PhAListener.OnBufferLevelEvent(ABufferStatus, 0);
    }
    
}
